#!/usr/bin/env python3
# -*- coding: utf-8 -*-

###################
#    This package implements an encoding to bypass entropy antivirus check.
#    Copyright (C) 2023  EntropyEncoding

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
###################

"""
This package implements an encoding to bypass entropy antivirus check.

https://www.schellman.com/blog/cybersecurity/penetration-testing-methods-entropy
https://vanmieghem.io/blueprint-for-evading-edr-in-2022/
https://pentest.blog/art-of-anti-detection-1-introduction-to-av-detection-techniques/

Information based on a blog (red teamer test a not named EDR):
    - antivirus detect potentially malicious entropy when the entropy score is greater than 7.2

~# python3 EntropyEncoding.py
Entropy for non-encoded secrets: 4.521591372417719
Entropy for non-encoded encrypted secrets: 7.945422222752084
Entropy for entropy-encoded encrypted secrets: 5.762166896848745
Entropy for entropy-encoded2 encrypted secrets: 5.748670434218312
Entropy for non-encoded exe: 5.22055339277441
Entropy for non-encoded encrypted exe: 7.923900258907012
Entropy for entropy-encoded encrypted exe: 5.756072685391074
Entropy for entropy-encoded2 encrypted exe: 5.799741821347019
~# 
"""

__version__ = "0.0.4"
__author__ = "Maurice Lambert"
__author_email__ = "mauricelambert434@gmail.com"
__maintainer__ = "Maurice Lambert"
__maintainer_email__ = "mauricelambert434@gmail.com"
__description__ = """
This package implements an encoding to bypass entropy antivirus check.
"""
__url__ = "https://github.com/mauricelambert/EntropyEncoding"

__all__ = [
    "entropy_encode2",
    "entropy_decode2",
    "entropy_encode",
    "entropy_decode",
    "shannon_entropy",
]

__license__ = "GPL-3.0 License"
__copyright__ = """
EntropyEncoding  Copyright (C) 2023  Maurice Lambert
This program comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions.
"""
copyright = __copyright__
license = __license__

print(copyright)

from math import log
from typing import Tuple, Dict
from random import choice, randint
from base64 import b32encode, b32decode
from collections import Counter, defaultdict

dll = (
    b'MZ\x90\x00\x03\x00\x00\x00\x04\x00\x00\x00\xff\xff\x00\x00\xb8\x00\x00\x00\x00\x00\x00\x00@\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x80\x00\x00\x00\x0e\x1f\xba\x0e\x00\xb4\t\xcd!\xb8\x01L\xcd!This program cannot be run in DOS mode.\r\r\n$\x00\x00\x00\x00\x00\x00\x00PE\x00\x00L\x01\x03\x00\xcf\x1e^\x92\x00\x00\x00\x00\x00\x00\x00\x00\xe0\x00\x02\x01\x0b\x010\x00\x00*\x00\x00\x00\n\x00\x00\x00\x00\x00\x00\x9eI\x00\x00\x00 \x00\x00\x00`\x00\x00\x00\x00@\x00\x00 \x00\x00\x00\x02\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00\x06\x00\x00\x00\x00\x00\x00\x00\x00\xa0\x00\x00\x00\x02\x00\x00\x85\xd9\x00\x00\x03\x00`\x85\x00\x00\x10\x00\x00\x10\x00\x00\x00\x00\x10\x00\x00\x10\x00\x00\x00\x00\x00\x00\x10\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00LI\x00\x00O\x00\x00\x00\x00`\x00\x00P\x07\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x80\x00\x00\x0c\x00\x00\x00\x80H\x00\x008\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00 \x00\x00\x08\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x08 \x00\x00H\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00.text\x00\x00\x00\xa4)\x00\x00\x00 \x00\x00\x00*\x00\x00\x00\x02\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00 \x00\x00`.rsrc\x00\x00\x00P\x07\x00\x00\x00`\x00\x00\x00\x08\x00\x00\x00,\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00@\x00\x00@.reloc\x00\x00\x0c\x00\x00\x00\x00\x80\x00\x00\x00\x02\x00\x00\x004\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00@\x00\x00B\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x80I\x00\x00\x00\x00\x00\x00H\x00\x00\x00\x02\x00\x05\x00\x04)\x00\x00\x84\x19\x00\x00\x03\x00\x00\x00\x05\x00\x00\x06\x88B\x00\x00\xf8\x05\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xfa\x02\x03(\x01\x00\x00+/ \x03\x02o\x15\x00\x00\no\x16\x00\x00\n\x171%\x03\x02o\x15\x00\x00\n\x16o\x17\x00\x00\n\x1f-3\x14\x03\x02\x17Yo\x15\x00\x00\n(\x0e\x00\x00\x06s\x18\x00\x00\nz*\x00\x130\x04\x00\x17\x03\x00\x00\x01\x00\x00\x11\x02-\x15r\x01\x00\x00p(\x19\x00\x00\no\x1a\x00\x00\ns\x1b\x00\x00\nzr\x0b\x00\x00p\x80\x02\x00\x00\x04r\x0b\x00\x00p\x80\x01\x00\x00\x04r\x0b\x00\x00p\x80\x04\x00\x00\x04r\x0b\x00\x00p\x80\x05\x00\x00\x04r\x0b\x00\x00p\x80\x07\x00\x00\x04r\x0b\x00\x00p\x80\x06\x00\x00\x04r\x0b\x00\x00p\x80\x03\x00\x00\x04r\x0b\x00\x00p\x80\x08\x00\x00\x04\x16\x80\n\x00\x00\x04\x02(\x01\x00\x00+\n\x16\x0b\x06:\x92\x02\x00\x00r\x0b\x00\x00ps\x1c\x00\x00\nz\x02\x07%\x17X\x0bo\x15\x00\x00\n(\x1d\x00\x00\no\x1e\x00\x00\n\x0c\x08o\x16\x00\x00\n\x161!\x08\x16o\x17\x00\x00\n\x1f-.\x16(\r\x00\x00\x06r\r\x00\x00p\x08(\x1f\x00\x00\ns\x1c\x00\x00\nz\x08(\x14\x00\x00\x06\r\t >\xa8kA58\t \n\x86\x8b\x135\x15\t c\xacN\x11.k\t \n\x86\x8b\x13.x8\x05\x02\x00\x00\t c\xfdf\x17;\x92\x00\x00\x00\t >\xa8kA;\xb1\x00\x00\x008\xea\x01\x00\x00\t \xf1\xf3\xd3V5\x18\t \xe7>\x16S;\xae\x00\x00\x00\t \xf1\xf3\xd3V.|8\xca\x01\x00\x00\t o7K\x8c;\xc0\x00\x00\x00\t \xb7\xc3X\xa0;\xa0\x00\x00\x00\t \x8c\x97\x7f\xa2./8\xa7\x01\x00\x00\x08r\x15\x00\x00p( \x00\x00\n:\xad\x00\x00\x008\x92\x01\x00\x00\x08r!\x00\x00p( \x00\x00\n:\xb4\x00\x00\x008}\x01\x00\x00\x08rA\x00\x00p( \x00\x00\n:\xbb\x00\x00\x008h\x01\x00\x00\x08r]\x00\x00p( \x00\x00\n:\xc2\x00\x00\x008S\x01\x00\x00\x08r\x87\x00\x00p( \x00\x00\n:\xc9\x00\x00\x008>\x01\x00\x00\x08r\x9b\x00\x00p( \x00\x00\n:\xcd\x00\x00\x008)\x01\x00\x00\x08r\xb3\x00\x00p( \x00\x00\n:\xd1\x00\x00\x008\x14\x01\x00\x00\x08r\xcd\x00\x00p( \x00\x00\n:\xda\x00\x00\x008\xff\x00\x00\x00\x08r\xe5\x00\x00p( \x00\x00\n:\xe3\x00\x00\x008\xea\x00\x00\x00\x07\x02(\x01\x00\x00\x06\x02\x07%\x17X\x0bo\x15\x00\x00\n\x80\x02\x00\x00\x048\xe4\x00\x00\x00\x07\x02(\x01\x00\x00\x06\x02\x07%\x17X\x0bo\x15\x00\x00\n\x80\x01\x00\x00\x048\xc8\x00\x00\x00\x07\x02(\x01\x00\x00\x06\x02\x07%\x17X\x0bo\x15\x00\x00\n\x80\x04\x00\x00\x048\xac\x00\x00\x00\x07\x02(\x01\x00\x00\x06\x02\x07%\x17X\x0bo\x15\x00\x00\n\x80\x05\x00\x00\x048\x90\x00\x00\x00\x07\x02(\x01\x00\x00\x06\x02\x07%\x17X\x0bo\x15\x00\x00\n\x80\x07\x00\x00\x04+w\x07\x02(\x01\x00\x00\x06\x02\x07%\x17X\x0bo\x15\x00\x00\n\x80\x06\x00\x00\x04+^\x07\x02(\x01\x00\x00\x06\x02\x07%\x17X\x0bo\x15\x00\x00\n(!\x00\x00\n\x80\x03\x00\x00\x04+@\x07\x02(\x01\x00\x00\x06\x02\x07%\x17X\x0bo\x15\x00\x00\n(!\x00\x00\n\x80\x08\x00\x00\x04+"\x07\x17X\x0b\x17\x80\n\x00\x00\x04+\x16(\r\x00\x00\x06r\r\x00\x00p\x08(\x1f\x00\x00\ns\x1c\x00\x00\nz\x07\x06?r\xfd\xff\xff*\x00\x130\x02\x00\x99\x00\x00\x00\x00\x00\x00\x00~\x01\x00\x00\x04("\x00\x00\n,\x10r\xf3\x00\x00p(\x0f\x00\x00\x06s\x18\x00\x00\nz~\x01\x00\x00\x04(#\x00\x00\n-\x10r\xf3\x00\x00p(\x0b\x00\x00\x06s\x18\x00\x00\nz~\x02\x00\x00\x04("\x00\x00\n,\x10r\x11\x01\x00p(\x0f\x00\x00\x06s\x18\x00\x00\nz~\x03\x00\x00\x04("\x00\x00\n,\x10r\x1b\x01\x00p(\x0f\x00\x00\x06s\x18\x00\x00\nz~\x07\x00\x00\x04("\x00\x00\n,\x1c~\x06\x00\x00\x04("\x00\x00\n,\x10r3\x01\x00p(\x0f\x00\x00\x06s\x18\x00\x00\nz*\x00\x00\x00\x1b0\x07\x00\xfd\x01\x00\x00\x02\x00\x00\x11~\x07\x00\x00\x04("\x00\x00\n-E~\x07\x00\x00\x04\x17\x8d+\x00\x00\x01%\x16\x1f,\x9do$\x00\x00\n~\x0e\x00\x00\x04%-\x17&~\r\x00\x00\x04\xfe\x06\x17\x00\x00\x06s%\x00\x00\n%\x80\x0e\x00\x00\x04(\x02\x00\x00+(\x03\x00\x00+s(\x00\x00\n+\x05s)\x00\x00\n\n~\x06\x00\x00\x04("\x00\x00\n-E~\x06\x00\x00\x04\x17\x8d+\x00\x00\x01%\x16\x1f,\x9do$\x00\x00\n~\x0f\x00\x00\x04%-\x17&~\r\x00\x00\x04\xfe\x06\x18\x00\x00\x06s%\x00\x00\n%\x80\x0f\x00\x00\x04(\x02\x00\x00+(\x03\x00\x00+s(\x00\x00\n+\x05s)\x00\x00\n\x0b\x06\x07(\x04\x00\x00+(\x05\x00\x00+\x0c\x08(\x06\x00\x00+,)(\x1d\x00\x00\n(\x12\x00\x00\x06\x17\x8d\x11\x00\x00\x01%\x16r[\x01\x00p\x08(-\x00\x00\n\xa2(.\x00\x00\ns/\x00\x00\nz~\x01\x00\x00\x04(!\x00\x00\n\x80\x01\x00\x00\x04s0\x00\x00\n\r\x06o1\x00\x00\n\x13\x04+Z\x12\x04(2\x00\x00\n\x13\x05~\x01\x00\x00\x04\x11\x05(3\x00\x00\n(4\x00\x00\n- (\x1d\x00\x00\n(\x11\x00\x00\x06\x17\x8d\x11\x00\x00\x01%\x16\x11\x05\xa2(.\x00\x00\ns/\x00\x00\nz\t~\x02\x00\x00\x04~\x01\x00\x00\x04\x11\x05(3\x00\x00\n\x14\x16s5\x00\x00\no6\x00\x00\n\x12\x04(7\x00\x00\n-\x9d\xde\x0e\x12\x04\xfe\x16\x05\x00\x00\x1bo8\x00\x00\n\xdc\x07o1\x00\x00\n\x13\x06+Z\x12\x06(2\x00\x00\n\x13\x07~\x01\x00\x00\x04\x11\x07(3\x00\x00\n(4\x00\x00\n- (\x1d\x00\x00\n(\x11\x00\x00\x06\x17\x8d\x11\x00\x00\x01%\x16\x11\x07\xa2(.\x00\x00\ns/\x00\x00\nz\t~\x02\x00\x00\x04~\x01\x00\x00\x04\x11\x07(3\x00\x00\n\x14\x17s5\x00\x00\no6\x00\x00\n\x12\x06(7\x00\x00\n-\x9d\xde\x0e\x12\x06\xfe\x16\x05\x00\x00\x1bo8\x00\x00\n\xdc\t*\x00\x00\x00\x01\x1c\x00\x00\x02\x00\t\x01gp\x01\x0e\x00\x00\x00\x00\x02\x00\x86\x01g\xed\x01\x0e\x00\x00\x00\x00\x1b0\x05\x00\xb8\x00\x00\x00\x03\x00\x00\x11\x02(\x02\x00\x00\x06(\x03\x00\x00\x06(\x04\x00\x00\x06\n~\x08\x00\x00\x04("\x00\x00\n-\'~\x08\x00\x00\x04(9\x00\x00\ns:\x00\x00\n\x0c\x08o;\x00\x00\n\x80\t\x00\x00\x04\xde\n\x08,\x06\x08o8\x00\x00\n\xdc~\x02\x00\x00\x04\x06~\t\x00\x00\x04~\x04\x00\x00\x04~\x05\x00\x00\x04s<\x00\x00\n\x0b\x07~\x03\x00\x00\x04o=\x00\x00\n\xdeK\r\to>\x00\x00\n(?\x00\x00\n(\x13\x00\x00\x06(?\x00\x00\n\xde3\x13\x04\x11\x04o>\x00\x00\n(?\x00\x00\n(\x13\x00\x00\x06(?\x00\x00\n\xde\x19\x13\x05~\n\x00\x00\x04,\x02\xfe\x1a\x11\x05o>\x00\x00\n(?\x00\x00\n\xde\x00*\x014\x00\x00\x02\x00-\x00\r:\x00\n\x00\x00\x00\x00\x00\x00\x00\x00ll\x00\x18\x19\x00\x00\x01\x00\x00\x00\x00l\x84\x00\x1a\x1a\x00\x00\x01\x00\x00\x00\x00l\x9e\x00\x19\x1b\x00\x00\x01\x1e\x02(@\x00\x00\n*\x130\x02\x00-\x00\x00\x00\x04\x00\x00\x11~\x0b\x00\x00\x04- r_\x01\x00p\xd0\x03\x00\x00\x02(A\x00\x00\noB\x00\x00\nsC\x00\x00\n\n\x06\x80\x0b\x00\x00\x04~\x0b\x00\x00\x04*\x1a~\x0c\x00\x00\x04*\x1e\x02\x80\x0c\x00\x00\x04*V(\x07\x00\x00\x06r\xdf\x01\x00p~\x0c\x00\x00\x04oD\x00\x00\n*V(\x07\x00\x00\x06r%\x02\x00p~\x0c\x00\x00\x04oD\x00\x00\n*V(\x07\x00\x00\x06rO\x02\x00p~\x0c\x00\x00\x04oD\x00\x00\n*V(\x07\x00\x00\x06r\x89\x02\x00p~\x0c\x00\x00\x04oD\x00\x00\n*V(\x07\x00\x00\x06r\xc7\x02\x00p~\x0c\x00\x00\x04oD\x00\x00\n*V(\x07\x00\x00\x06r\x01\x03\x00p~\x0c\x00\x00\x04oD\x00\x00\n*V(\x07\x00\x00\x06r7\x03\x00p~\x0c\x00\x00\x04oD\x00\x00\n*V(\x07\x00\x00\x06rw\x03\x00p~\x0c\x00\x00\x04oD\x00\x00\n*V(\x07\x00\x00\x06r\xa9\x03\x00p~\x0c\x00\x00\x04oD\x00\x00\n*V(\x07\x00\x00\x06r\x01\x04\x00p~\x0c\x00\x00\x04oD\x00\x00\n*\x130\x02\x00,\x00\x00\x00\x05\x00\x00\x11\x02,\' \xc5\x9d\x1c\x81\n\x16\x0b+\x14\x02\x07o\x17\x00\x00\n\x06a \x93\x01\x00\x01Z\n\x07\x17X\x0b\x07\x02o\x16\x00\x00\n2\xe3\x06*.s\x16\x00\x00\x06\x80\r\x00\x00\x04*\x1e\x02(@\x00\x00\n*\x1e\x03oE\x00\x00\n*\x1e\x03oE\x00\x00\n*BSJB\x01\x00\x01\x00\x00\x00\x00\x00\x0c\x00\x00\x00v4.0.30319\x00\x00\x00\x00\x05\x00l\x00\x00\x000\x07\x00\x00#~\x00\x00\x9c\x07\x00\x00\xe4\t\x00\x00#Strings\x00\x00\x00\x00\x80\x11\x00\x00\x18\x04\x00\x00#US\x00\x98\x15\x00\x00\x10\x00\x00\x00#GUID\x00\x00\x00\xa8\x15\x00\x00\xdc\x03\x00\x00#Blob\x00\x00\x00\x00\x00\x00\x00\x02\x00\x00\x01W\x15\xa2\t\t\x0b\x00\x00\x00\xfa\x013\x00\x16\x00\x00\x01\x00\x00\x006\x00\x00\x00\x05\x00\x00\x00\x0f\x00\x00\x00\x18\x00\x00\x00\x08\x00\x00\x00E\x00\x00\x00\x16\x00\x00\x00\x05\x00\x00\x00\x01\x00\x00\x00\x0c\x00\x00\x00\r\x00\x00\x00\x05\x00\x00\x00\x01\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x01\x00\x00\x00\x06\x00\x00\x00\x00\x00l\x05\x01\x00\x00\x00\x00\x00\x06\x00\xbc\x04\x1a\x08\x06\x00?\x05\x1a\x08\x06\x00\xef\x03\xe8\x07\x0f\x00\x95\x08\x00\x00\x06\x00\x83\x04\xca\x06\x06\x00d\x04\xca\x06\x06\x000\x04\xca\x06\x06\x00\xcd\x03:\x08\x06\x00\xa0\x04\x1a\x08\x06\x00\x8b\x03\xfb\x07\x06\x00\x1c\x04\xfb\x07\x06\x00\x10\x05\x7f\x06\x06\x00\xdc\x04\xca\x06\x06\x00\xf5\x04\xca\x06\x06\x00&\x05\xca\x06\x06\x00K\x04\x96\x05\x06\x00J\t\x7f\x06\x06\x00{\x00#\x02\x06\x00|\x00#\x02\n\x00m\x06?\x06\x0e\x00q\x00#\x02W\x00\xa5\x07\x00\x00\n\x00`\x06?\x06\x06\x007\x07\xb2\x01\x06\x00\x08\x07\x7f\x06\n\x00\xdc\x06?\x06\x06\x00\x10\x07\x7f\x06\x06\x00p\x03\x1a\x08\x06\x00\x83\x00\x7f\x06\x12\x00\x99\x03\x82\x07\x06\x00\xb0\x03\xe8\x07\x06\x00r\x07:\x08\x06\x00\x1a\x07\xa1\x06\x12\x00[\x03\x14\x06\x12\x00\x03\x04\x14\x06\x0e\x00\x7f\x02&\x07\x06\x00c\x00#\x02\x06\x00\xbc\x05\x7f\x06\x06\x002\x03\xca\x06\x06\x00\xf2\x06\x7f\x06\x06\x00\x04\x06\xb2\x01\x06\x00\xbf\t\xb2\x01\x06\x002\x07\x7f\x06\x06\x00O\x07\x7f\x06\x06\x00\xba\x02\xb2\x01\n\x00\xb6\x06?\x06\x06\x00\x8a\x02\x7f\x06\x06\x00\x8d\x05\x91\t\x06\x00D\x07\xb2\x01\n\x00=\x03?\x06\x06\x00\xbf\x02\x7f\x06\x06\x00\xe6\x02\x7f\x06\x06\x00\x96\x02\x7f\x06\x06\x00\xb2\t\xca\x06\x00\x00\x00\x00\x97\x00\x00\x00\x00\x00\x01\x00\x01\x00\x81\x01\x10\x00\xc9\x07\xb0\x07E\x00\x01\x00\x01\x00\x00\x00\x10\x00A\x08\xa4\x08E\x00\x0b\x00\x06\x00\x00\x01\x00\x00\xa0\x00\x00\x00E\x00\r\x00\x14\x00\x03!\x10\x00\x1f\x02\x00\x00E\x00\r\x00\x15\x00\x11\x00_\x07\xde\x01\x11\x00j\x02\xde\x01\x11\x00\xe6\x05\xde\x01\x11\x00\xc7\x02\xde\x01\x11\x00>\x02\xde\x01\x11\x00\xf9\x08\xde\x01\x11\x00\xfb\x08\xde\x01\x11\x00\xfe\x05\xde\x01\x11\x00z\t\xde\x01\x11\x00\xce\x05\xe1\x01\x11\x00\x8b\x06\xe4\x01\x11\x00\x0f\x03\xe9\x016\x00\x93\x00\xee\x01\x16\x00\x01\x00\xf2\x01\x16\x002\x00\xf2\x01P \x00\x00\x00\x00\x91\x00]\x05\xfa\x01\x01\x00\x90 \x00\x00\x00\x00\x93\x00\xef\x08\x04\x02\x03\x00\xb4#\x00\x00\x00\x00\x93\x00\xda\x08\r\x02\x04\x00\\$\x00\x00\x00\x00\x93\x00\r\t\x11\x02\x04\x00\x84&\x00\x00\x00\x00\x96\x00\x97\x06\x1a\x02\x04\x00|\'\x00\x00\x00\x00\x83\x18\xdb\x07\x06\x00\x05\x00\x84\'\x00\x00\x00\x00\x93\x08n\x07 \x02\x05\x00\xbd\'\x00\x00\x00\x00\x93\x08\xf7\x02]\x00\x05\x00\xc4\'\x00\x00\x00\x00\x93\x08\x03\x03&\x02\x05\x00\xcc\'\x00\x00\x00\x00\x93\x08\xbf\x00-\x02\x06\x00\xe2\'\x00\x00\x00\x00\x93\x08\xe6\x00-\x02\x06\x00\xf8\'\x00\x00\x00\x00\x93\x08\xec\x01-\x02\x06\x00\x0e(\x00\x00\x00\x00\x93\x08\x8f\x01-\x02\x06\x00$(\x00\x00\x00\x00\x93\x08O\x01-\x02\x06\x00:(\x00\x00\x00\x00\x93\x08p\x01-\x02\x06\x00P(\x00\x00\x00\x00\x93\x08\x1c\x01-\x02\x06\x00f(\x00\x00\x00\x00\x93\x08\xff\x00-\x02\x06\x00|(\x00\x00\x00\x00\x93\x08\xbc\x01-\x02\x06\x00\x92(\x00\x00\x00\x00\x93\x08@\x01-\x02\x06\x00\xa8(\x00\x00\x00\x00\x93\x00\xd4\x051\x02\x06\x00\xe0(\x00\x00\x00\x00\x91\x18\xe1\x07\r\x02\x07\x00\xec(\x00\x00\x00\x00\x86\x18\xdb\x07\x06\x00\x07\x00\xf4(\x00\x00\x00\x00\x83\x00\x0b\x006\x02\x07\x00\xfc(\x00\x00\x00\x00\x83\x00<\x006\x02\x08\x00\x00\x00\x01\x00\x9d\t\x00\x00\x02\x00\x08\t\x00\x00\x01\x00\x08\t\x00\x00\x01\x00\x08\t\x00\x00\x01\x00f\x05\x00\x00\x01\x00:\t\x00\x00\x01\x00\xc3\x05\x00\x00\x01\x00\xc3\x05\t\x00\xdb\x07\x01\x00\x11\x00\xdb\x07\x06\x00\x19\x00\xdb\x07\n\x00)\x00\xdb\x07\x10\x001\x00\xdb\x07\x10\x009\x00\xdb\x07\x10\x00A\x00\xdb\x07\x10\x00I\x00\xdb\x07\x10\x00Q\x00\xdb\x07\x10\x00Y\x00\xdb\x07\x15\x00a\x00\xdb\x07\x15\x00i\x00\xdb\x07\x10\x00q\x00\xdb\x07\x10\x00y\x00\xdb\x07\x10\x00\x81\x00\xdb\x07\x10\x00\xe1\x00\xdb\x07\x06\x00\xf1\x00\xdb\x07\x1a\x00\xf9\x00\xdb\x07\x06\x00\x19\x01\xdb\x07 \x00!\x01t\t\'\x00\x0c\x00W\x06=\x001\x01\t\x06C\x001\x01+\tG\x00\xd1\x00\xdb\x07\x1a\x009\x01Y\x02S\x00\x89\x00\xb0\x05Y\x00A\x01\xdb\x07\x1a\x00\xd1\x00\xdb\x07\x10\x00\t\x01\x1f\x03]\x001\x01\x9a\x07c\x001\x01<\tj\x001\x01\xc9\tq\x00I\x01\xf2\x05w\x001\x01\xd5\t|\x00Q\x015\t|\x001\x01b\t\xa5\x00\x14\x00\xdb\x07\xb3\x00!\x01Q\t\xb9\x00!\x01\xa6\t\xd8\x00\x1c\x00\xdb\x07\xec\x00\x1c\x00\xdb\x07\x06\x00!\x01X\t\xf7\x00!\x01\x81\t\x10\x01!\x01\xbb\t!\x011\x01\x9c\x06-\x011\x01C\t8\x01\xc9\x00\xdb\x07\x10\x00$\x00\xdb\x07\x06\x00\x1c\x00\xa2\x07I\x01,\x00h\tX\x01I\x01\xde\x02]\x01i\x015\t|\x00\xa1\x00\xdb\x07c\x01$\x00K\x02m\x01,\x00\x88\ts\x01y\x01S\x03\x06\x00\x81\x01\x8a\x00\x8a\x01\xc1\x00\xdb\x07\x90\x01\x89\x01O\x02Y\x00\xb9\x00\xdb\x07\x98\x01\x91\x01\r\x02\x10\x00\xd9\x00s\x02Y\x00\x99\x01\xd4\x02\xa6\x01\x89\x00\xdb\x07\x06\x00\xa1\x01\xa8\x02\xb1\x01\xa1\x01\xae\t\xba\x01\x01\x01\xdb\x07\xc0\x01\x01\x01\xb9\x05\xc8\x011\x01\x86\x06Y\x00)\x00\x9b\x00s\x02.\x00\x0b\x00K\x02.\x00\x13\x00T\x02.\x00\x1b\x00s\x02.\x00#\x00|\x02.\x00+\x00|\x02.\x003\x00|\x02.\x00;\x00\x82\x02.\x00C\x00\x8d\x02.\x00K\x00\xa3\x02.\x00S\x00|\x02.\x00[\x00\xcd\x02.\x00c\x00\xd3\x02.\x00k\x00\x03\x03.\x00s\x00B\x03.\x00{\x00]\x03I\x00\x9b\x00s\x02c\x00\x8b\x00\x95\x03c\x00\x93\x00\xd6\x03c\x00\x83\x00\xd6\x03\x83\x00\x83\x00\xd6\x03\xa3\x00\x83\x00\xd6\x03L\x00\x81\x00w\x01\xab\x01\xd0\x01\x03\x00\x01\x00\x00\x00r\x07;\x02\x00\x00*\x03A\x02\x00\x00\xc3\x00G\x02\x00\x00\xea\x00G\x02\x00\x00\xf0\x01G\x02\x00\x00\x93\x01G\x02\x00\x00S\x01G\x02\x00\x00t\x01G\x02\x00\x00 \x01G\x02\x00\x00\x03\x01G\x02\x00\x00\xc0\x01G\x02\x00\x00D\x01G\x02\x02\x00\x07\x00\x03\x00\x02\x00\x08\x00\x05\x00\x01\x00\t\x00\x05\x00\x02\x00\n\x00\x07\x00\x02\x00\x0b\x00\t\x00\x02\x00\x0c\x00\x0b\x00\x02\x00\r\x00\r\x00\x02\x00\x0e\x00\x0f\x00\x02\x00\x0f\x00\x11\x00\x02\x00\x10\x00\x13\x00\x02\x00\x11\x00\x15\x00\x02\x00\x12\x00\x17\x00\x02\x00\x13\x00\x19\x007\x00\xac\x00\xe6\x00B\x01R\x01\x04\x80\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xbe\x07\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xd5\x01\x16\x02\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00*\x06\x00\x00\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xd5\x01\xeb\x02\x00\x00\x00\x00\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xd5\x01\x7f\x06\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00K\x08\x00\x00\x05\x00\x02\x00)\x003\x00M\x00\xd3\x00O\x003\x00U\x003\x00W\x003\x00Y\x003\x00\x00\x00\x00\x00\x00<>9__13_0\x00<GetTemplateConfigurationItems>b__13_0\x00<>9__13_1\x00<GetTemplateConfigurationItems>b__13_1\x00IEnumerable`1\x00HashSet`1\x00IList`1\x00Func`2\x00get_UTF8\x00<>9\x00<Module>\x00<PrivateImplementationDetails>\x00get_ERROR_AT_LEAST_ONE_ACTION_REQUIRED\x00get_ERROR_FILE_NOT_FOUND\x00get_ERROR_TEMPLATE_NOT_FOUND\x00get_ERROR_TEMPLATE_FOLDER_NOT_FOUND\x00get_TEXT_USAGE\x00get_ERROR_MISSING_ARGUMENT_VALUE\x00get_ERROR_MISSING_REQUIRED_ARG\x00get_ERROR_INVALID_COMMAND_LINE_ARG\x00System.IO\x00get_ERROR_TEMPLATE_REGISTER_UNREGISTER_CONFLICT\x00get_ERROR_INVALID_ARGUMENT_COUNT\x00WriteCab\x00mscorlib\x00<>c\x00System.Collections.Generic\x00baselineDesc\x00Add\x00ReadToEnd\x00GetCurrentMethod\x00siteCode\x00get_Message\x00Enumerable\x00IDisposable\x00RuntimeTypeHandle\x00GetTypeFromHandle\x00File\x00Console\x00baselineName\x00WriteLine\x00Combine\x00Type\x00System.Core\x00get_Culture\x00set_Culture\x00resourceCulture\x00get_CurrentCulture\x00MethodBase\x00ConfigurationItemBase\x00Dispose\x00EditorBrowsableState\x00CompilerGeneratedAttribute\x00GuidAttribute\x00GeneratedCodeAttribute\x00DebuggerNonUserCodeAttribute\x00NeutralResourcesLanguageAttribute\x00DebuggableAttribute\x00EditorBrowsableAttribute\x00ComVisibleAttribute\x00AssemblyTrademarkAttribute\x00TargetFrameworkAttribute\x00AssemblyConfigurationAttribute\x00AssemblyDescriptionAttribute\x00InternalsVisibleToAttribute\x00CompilationRelaxationsAttribute\x00AssemblyProductAttribute\x00AssemblyCopyrightAttribute\x00CLSCompliantAttribute\x00AssemblyCompanyAttribute\x00RuntimeCompatibilityAttribute\x00HasValue\x00value\x00UevTemplateBaselineGenerator.exe\x00Encoding\x00System.Runtime.Versioning\x00ToString\x00GetString\x00trimmedArg\x00debug\x00ComputeStringHash\x00cabFilePath\x00GetFullPath\x00digestPath\x00get_Length\x00System.ComponentModel\x00Microsoft.Uev.CmUtil\x00Microsoft.Uev.UevCmUtil\x00get_Item\x00BaselineItem\x00ConfigurationItem\x00System\x00Trim\x00resourceMan\x00Main\x00Join\x00System.Globalization\x00ConfigurationAction\x00System.Reflection\x00CmCmdLineArgException\x00ArgumentNullException\x00ArgumentException\x00CultureInfo\x00System.Linq\x00Char\x00StreamReader\x00TextReader\x00IFormatProvider\x00templateFolder\x00get_ResourceManager\x00System.CodeDom.Compiler\x00ToLower\x00GetEnumerator\x00Microsoft.Uev.UevTemplateBaselineGenerator\x00.ctor\x00.cctor\x00System.Diagnostics\x00System.Runtime.InteropServices\x00System.Runtime.CompilerServices\x00System.Resources\x00Microsoft.Uev.UevTemplateBaselineGenerator.Properties.Resources.resources\x00DebuggingModes\x00Microsoft.Uev.UevTemplateBaselineGenerator.Properties\x00ValidateRequiredArgs\x00ParseArgs\x00unregisterArgs\x00args\x00GetTemplateConfigurationItems\x00get_Chars\x00Exists\x00Concat\x00Format\x00Object\x00Select\x00Intersect\x00Split\x00get_Current\x00Count\x00digest\x00ToList\x00MoveNext\x00System.Text\x00argIndex\x00ToArray\x00get_Assembly\x00Any\x00Directory\x00op_Equality\x00IsNullOrEmpty\x00\x00\x00\ta\x00r\x00g\x00s\x00\x00\x01\x00\x07 \x00:\x00 \x00\x00\x0b-\x00s\x00i\x00t\x00e\x00\x01\x1f-\x00t\x00e\x00m\x00p\x00l\x00a\x00t\x00e\x00f\x00o\x00l\x00d\x00e\x00r\x00\x01\x1b-\x00b\x00a\x00s\x00e\x00l\x00i\x00n\x00e\x00n\x00a\x00m\x00e\x00\x01)-\x00b\x00a\x00s\x00e\x00l\x00i\x00n\x00e\x00d\x00e\x00s\x00c\x00r\x00i\x00p\x00t\x00i\x00o\x00n\x00\x01\x13-\x00r\x00e\x00g\x00i\x00s\x00t\x00e\x00r\x00\x01\x17-\x00u\x00n\x00r\x00e\x00g\x00i\x00s\x00t\x00e\x00r\x00\x01\x19-\x00c\x00a\x00b\x00f\x00i\x00l\x00e\x00p\x00a\x00t\x00h\x00\x01\x17-\x00d\x00i\x00g\x00e\x00s\x00t\x00p\x00a\x00t\x00h\x00\x01\r-\x00d\x00e\x00b\x00u\x00g\x00\x01\x1dT\x00e\x00m\x00p\x00l\x00a\x00t\x00e\x00F\x00o\x00l\x00d\x00e\x00r\x00\x00\tS\x00i\x00t\x00e\x00\x00\x17C\x00a\x00b\x00F\x00i\x00l\x00e\x00P\x00a\x00t\x00h\x00\x00\'R\x00e\x00g\x00i\x00s\x00t\x00e\x00r\x00|\x00U\x00n\x00r\x00e\x00g\x00i\x00s\x00t\x00e\x00r\x00\x00\x03,\x00\x00\x7fM\x00i\x00c\x00r\x00o\x00s\x00o\x00f\x00t\x00.\x00U\x00e\x00v\x00.\x00U\x00e\x00v\x00T\x00e\x00m\x00p\x00l\x00a\x00t\x00e\x00B\x00a\x00s\x00e\x00l\x00i\x00n\x00e\x00G\x00e\x00n\x00e\x00r\x00a\x00t\x00o\x00r\x00.\x00P\x00r\x00o\x00p\x00e\x00r\x00t\x00i\x00e\x00s\x00.\x00R\x00e\x00s\x00o\x00u\x00r\x00c\x00e\x00s\x00\x00EE\x00R\x00R\x00O\x00R\x00_\x00A\x00T\x00_\x00L\x00E\x00A\x00S\x00T\x00_\x00O\x00N\x00E\x00_\x00A\x00C\x00T\x00I\x00O\x00N\x00_\x00R\x00E\x00Q\x00U\x00I\x00R\x00E\x00D\x00\x00)E\x00R\x00R\x00O\x00R\x00_\x00F\x00I\x00L\x00E\x00_\x00N\x00O\x00T\x00_\x00F\x00O\x00U\x00N\x00D\x00\x009E\x00R\x00R\x00O\x00R\x00_\x00I\x00N\x00V\x00A\x00L\x00I\x00D\x00_\x00A\x00R\x00G\x00U\x00M\x00E\x00N\x00T\x00_\x00C\x00O\x00U\x00N\x00T\x00\x00=E\x00R\x00R\x00O\x00R\x00_\x00I\x00N\x00V\x00A\x00L\x00I\x00D\x00_\x00C\x00O\x00M\x00M\x00A\x00N\x00D\x00_\x00L\x00I\x00N\x00E\x00_\x00A\x00R\x00G\x00\x009E\x00R\x00R\x00O\x00R\x00_\x00M\x00I\x00S\x00S\x00I\x00N\x00G\x00_\x00A\x00R\x00G\x00U\x00M\x00E\x00N\x00T\x00_\x00V\x00A\x00L\x00U\x00E\x00\x005E\x00R\x00R\x00O\x00R\x00_\x00M\x00I\x00S\x00S\x00I\x00N\x00G\x00_\x00R\x00E\x00Q\x00U\x00I\x00R\x00E\x00D\x00_\x00A\x00R\x00G\x00\x00?E\x00R\x00R\x00O\x00R\x00_\x00T\x00E\x00M\x00P\x00L\x00A\x00T\x00E\x00_\x00F\x00O\x00L\x00D\x00E\x00R\x00_\x00N\x00O\x00T\x00_\x00F\x00O\x00U\x00N\x00D\x00\x001E\x00R\x00R\x00O\x00R\x00_\x00T\x00E\x00M\x00P\x00L\x00A\x00T\x00E\x00_\x00N\x00O\x00T\x00_\x00F\x00O\x00U\x00N\x00D\x00\x00WE\x00R\x00R\x00O\x00R\x00_\x00T\x00E\x00M\x00P\x00L\x00A\x00T\x00E\x00_\x00R\x00E\x00G\x00I\x00S\x00T\x00E\x00R\x00_\x00U\x00N\x00R\x00E\x00G\x00I\x00S\x00T\x00E\x00R\x00_\x00C\x00O\x00N\x00F\x00L\x00I\x00C\x00T\x00\x00\x15T\x00E\x00X\x00T\x00_\x00U\x00S\x00A\x00G\x00E\x00\x00\x00\x98\t\xc2b*\x80\xf5@\x96\x1f\xe7\x1b\xa6\x93q\xbe\x00\x04 \x01\x01\x08\x03 \x00\x01\x05 \x01\x01\x11\x11\x04 \x01\x01\x0e\x04 \x01\x01\x02\x05 \x02\x01\x0e\x0e\x06 \x01\x01\x11\x80\x89\x0b\x10\x01\x01\x08\x15\x12\x80\x95\x01\x1e\x00\x03\n\x01\x0e\x05\x15\x12I\x01\x0e\x05 \x01\x13\x00\x08\x03 \x00\x08\x04 \x01\x03\x08\x06\x07\x04\x08\x08\x0e\t\x05\x00\x00\x12\x80\x9d\x03 \x00\x0e\x05\x00\x00\x12\x80\x85\x06 \x01\x0e\x12\x80\x85\x06\x00\x03\x0e\x0e\x0e\x0e\x05\x00\x02\x02\x0e\x0e\x04\x00\x01\x0e\x0e\x04\x00\x01\x02\x0e#\x07\x08\x15\x12U\x01\x0e\x15\x12U\x01\x0e\x15\x12M\x01\x0e\x15\x12M\x01\x12Q\x15\x11Y\x01\x0e\x0e\x15\x11Y\x01\x0e\x0e\x06 \x01\x1d\x0e\x1d\x03\x06\x15\x12u\x02\x0e\x0e\x05 \x02\x01\x1c\x18\x19\x10\x02\x02\x15\x12\x80\x95\x01\x1e\x01\x15\x12\x80\x95\x01\x1e\x00\x15\x12u\x02\x1e\x00\x1e\x01\x04\n\x02\x0e\x0e\r\x10\x01\x01\x1d\x1e\x00\x15\x12\x80\x95\x01\x1e\x00\x05\x15\x12U\x01\x0e\n \x01\x01\x15\x12\x80\x95\x01\x13\x00\x18\x10\x01\x02\x15\x12\x80\x95\x01\x1e\x00\x15\x12\x80\x95\x01\x1e\x00\x15\x12\x80\x95\x01\x1e\x00\x10\x10\x01\x01\x15\x12M\x01\x1e\x00\x15\x12\x80\x95\x01\x1e\x00\x0b\x10\x01\x01\x02\x15\x12\x80\x95\x01\x1e\x00\n\x00\x02\x0e\x0e\x15\x12\x80\x95\x01\x0e\t\x00\x03\x0e\x12\x80\xb1\x0e\x1d\x1c\x06\x15\x12M\x01\x12Q\x08 \x00\x15\x11Y\x01\x13\x00\x05\x15\x11Y\x01\x0e\x04 \x00\x13\x00\x05\x00\x02\x0e\x0e\x0e\t \x04\x01\x0e\x0e\x0e\x11\x80\xb9\x05 \x01\x01\x13\x00\x03 \x00\x02\x12\x07\x06\x15\x12M\x01\x12Q\x12]\x12a\x12e\x12i\x12m\x05\x00\x00\x12\x80\xc1\x07 \x02\x01\x0e\x12\x80\xc1\r \x05\x01\x0e\x15\x12M\x01\x12Q\x0e\x0e\x0e\x04\x00\x01\x01\x0e\x05\x07\x01\x12\x80\x81\x08\x00\x01\x12\x80\xd1\x11\x80\xd5\x05 \x00\x12\x80\xd9\x07 \x02\x01\x0e\x12\x80\xd9\x07 \x02\x0e\x0e\x12\x80\x85\x04\x07\x02\t\x08\x08\xb7z\\V\x194\xe0\x89\x02\x06\x0e\x02\x06\x02\x04\x06\x12\x80\x81\x04\x06\x12\x80\x85\x03\x06\x12\x14\x07\x06\x15\x12u\x02\x0e\x0e\t\x00\x02\x01\x08\x15\x12I\x01\x0e\x08\x00\x01\x01\x15\x12I\x01\x0e\x03\x00\x00\x01\x08\x00\x00\x15\x12M\x01\x12Q\x05\x00\x01\x01\x1d\x0e\x05\x00\x00\x12\x80\x81\x06\x00\x01\x01\x12\x80\x85\x03\x00\x00\x0e\x04\x00\x01\t\x0e\x04 \x01\x0e\x0e\x05\x08\x00\x12\x80\x81\x05\x08\x00\x12\x80\x85\x03\x08\x00\x0e\x08\x01\x00\x08\x00\x00\x00\x00\x00\x1e\x01\x00\x01\x00T\x02\x16WrapNonExceptionThrows\x01\x08\x01\x00\x02\x00\x00\x00\x00\x00\x05\x01\x00\x00\x00\x00\n\x01\x00\x05en-US\x00\x00\x15\x01\x00\x10UevCmUtil.uev.ut\x00\x00)\x01\x00$44e0f318-5faa-47f9-8ad3-1775aca4c017\x00\x00\x05\x01\x00\x01\x00\x00/\x01\x00*Microsoft (R) Windows (R) Operating System\x00\x00>\x01\x009Copyright (c) Microsoft Corporation. All rights reserved.\x00\x00\x1a\x01\x00\x15Microsoft Corporation\x00\x007\x01\x00\x1a.NETFramework,Version=v4.5\x01\x00T\x0e\x14FrameworkDisplayName\x00@\x01\x003System.Resources.Tools.StronglyTypedResourceBuilder\x074.0.0.0\x00\x00\x04\x01\x00\x00\x00\x00\xef\x05\x00\x00\xce\xca\xef\xbe\x01\x00\x00\x00\x91\x00\x00\x00lSystem.Resources.ResourceReader, mscorlib, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089#System.Resources.RuntimeResourceSet\x02\x00\x00\x00\n\x00\x00\x00\x00\x00\x00\x00PADPADP#v\x9f\x89\xe2NJ\x8c8\x92\xa4\x8e\xe7\xfcY\xa7\xfb\xab\xdf\xfd\xd1nP9Q\x92o<\x80\x10DE\xfe\x19\x82]K\xe8Vsv\x00\x00\x00=\x02\x00\x00j\x01\x00\x00I\x00\x00\x001\x01\x00\x00\xb3\x00\x00\x00\xad\x01\x00\x00\xe2\x01\x00\x00\xf4\x00\x00\x00\x00\x00\x00\x00Z\x03\x00\x00DE\x00R\x00R\x00O\x00R\x00_\x00A\x00T\x00_\x00L\x00E\x00A\x00S\x00T\x00_\x00O\x00N\x00E\x00_\x00A\x00C\x00T\x00I\x00O\x00N\x00_\x00R\x00E\x00Q\x00U\x00I\x00R\x00E\x00D\x00\x00\x00\x00\x00(E\x00R\x00R\x00O\x00R\x00_\x00F\x00I\x00L\x00E\x00_\x00N\x00O\x00T\x00_\x00F\x00O\x00U\x00N\x00D\x00;\x00\x00\x008E\x00R\x00R\x00O\x00R\x00_\x00I\x00N\x00V\x00A\x00L\x00I\x00D\x00_\x00A\x00R\x00G\x00U\x00M\x00E\x00N\x00T\x00_\x00C\x00O\x00U\x00N\x00T\x00K\x00\x00\x00<E\x00R\x00R\x00O\x00R\x00_\x00I\x00N\x00V\x00A\x00L\x00I\x00D\x00_\x00C\x00O\x00M\x00M\x00A\x00N\x00D\x00_\x00L\x00I\x00N\x00E\x00_\x00A\x00R\x00G\x00\x88\x00\x00\x008E\x00R\x00R\x00O\x00R\x00_\x00M\x00I\x00S\x00S\x00I\x00N\x00G\x00_\x00A\x00R\x00G\x00U\x00M\x00E\x00N\x00T\x00_\x00V\x00A\x00L\x00U\x00E\x00\x9a\x00\x00\x004E\x00R\x00R\x00O\x00R\x00_\x00M\x00I\x00S\x00S\x00I\x00N\x00G\x00_\x00R\x00E\x00Q\x00U\x00I\x00R\x00E\x00D\x00_\x00A\x00R\x00G\x00\xd1\x00\x00\x00>E\x00R\x00R\x00O\x00R\x00_\x00T\x00E\x00M\x00P\x00L\x00A\x00T\x00E\x00_\x00F\x00O\x00L\x00D\x00E\x00R\x00_\x00N\x00O\x00T\x00_\x00F\x00O\x00U\x00N\x00D\x00\xf0\x00\x00\x000E\x00R\x00R\x00O\x00R\x00_\x00T\x00E\x00M\x00P\x00L\x00A\x00T\x00E\x00_\x00N\x00O\x00T\x00_\x00F\x00O\x00U\x00N\x00D\x00\x15\x01\x00\x00VE\x00R\x00R\x00O\x00R\x00_\x00T\x00E\x00M\x00P\x00L\x00A\x00T\x00E\x00_\x00R\x00E\x00G\x00I\x00S\x00T\x00E\x00R\x00_\x00U\x00N\x00R\x00E\x00G\x00I\x00S\x00T\x00E\x00R\x00_\x00C\x00O\x00N\x00F\x00L\x00I\x00C\x00T\x008\x01\x00\x00\x14T\x00E\x00X\x00T\x00_\x00U\x00S\x00A\x00G\x00E\x00\x82\x01\x00\x00\x019At least one template must be registered or unregistered.\x01\x0eFile not found\x01;Invalid argument count. Possibly missing an agrument value.\x01\x10Invalid argument\x015Command line argument is missing the argument value\r\n\x01\x1dRequired argument not found\r\n\x01#UE-V Template folder not found: {0}\x01!UE-V template file not found: {0}\x01HOne or more templates are marked for both register and unregister: {0}\r\n\x01\x90\x02Usage: UevTemplateBaselineGenerator -Site <CM Site Code> [-BaselineName <name>] [-BaselineDescription <description>] -TemplateFolder <UE-V Template Folder Path> [-Register <template1,template2,...>] [-Unregister <template1,template2,...>] -CabFilePath <full path\\file.cab>\x00\x00\x00\x00\x00\x00\x00\x00\x00\x92t\xa8\xb0\x00\x00\x00\x00\x02\x00\x00\x009\x00\x00\x00\xb8H\x00\x00\xb8*\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x10\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00RSDS\x15\xb4T\xc7\x1b\xd8\x8fC\xab\xd9A\xe4\xaf\x89n\xa2\x01\x00\x00\x00UevTemplateBaselineGenerator.pdb\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00tI\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x8eI\x00\x00\x00 \x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x80I\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00_CorExeMain\x00mscoree.dll\x00\x00\x00\x00\x00\xff%\x00 @\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x02\x00\x10\x00\x00\x00 \x00\x00\x80\x18\x00\x00\x00P\x00\x00\x80\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x01\x00\x00\x008\x00\x00\x80\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\x80\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x01\x00\x00\x00h\x00\x00\x80\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\x00\xa8\x04\x00\x00\x90`\x00\x00\x18\x04\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x18\x044\x00\x00\x00V\x00S\x00_\x00V\x00E\x00R\x00S\x00I\x00O\x00N\x00_\x00I\x00N\x00F\x00O\x00\x00\x00\x00\x00\xbd\x04\xef\xfe\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00?\x00\x00\x00\x00\x00\x00\x00\x04\x00\x00\x00\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00D\x00\x00\x00\x01\x00V\x00a\x00r\x00F\x00i\x00l\x00e\x00I\x00n\x00f\x00o\x00\x00\x00\x00\x00$\x00\x04\x00\x00\x00T\x00r\x00a\x00n\x00s\x00l\x00a\x00t\x00i\x00o\x00n\x00\x00\x00\x00\x00\x00\x00\xb0\x04x\x03\x00\x00\x01\x00S\x00t\x00r\x00i\x00n\x00g\x00F\x00i\x00l\x00e\x00I\x00n\x00f\x00o\x00\x00\x00T\x03\x00\x00\x01\x000\x000\x000\x000\x000\x004\x00b\x000\x00\x00\x00\x1a\x00\x01\x00\x01\x00C\x00o\x00m\x00m\x00e\x00n\x00t\x00s\x00\x00\x00\x00\x00\x00\x00L\x00\x16\x00\x01\x00C\x00o\x00m\x00p\x00a\x00n\x00y\x00N\x00a\x00m\x00e\x00\x00\x00\x00\x00M\x00i\x00c\x00r\x00o\x00s\x00o\x00f\x00t\x00 \x00C\x00o\x00r\x00p\x00o\x00r\x00a\x00t\x00i\x00o\x00n\x00\x00\x00,\x00\x02\x00\x01\x00F\x00i\x00l\x00e\x00D\x00e\x00s\x00c\x00r\x00i\x00p\x00t\x00i\x00o\x00n\x00\x00\x00\x00\x00 \x00\x00\x000\x00\x08\x00\x01\x00F\x00i\x00l\x00e\x00V\x00e\x00r\x00s\x00i\x00o\x00n\x00\x00\x00\x00\x000\x00.\x000\x00.\x000\x00.\x000\x00\x00\x00b\x00!\x00\x01\x00I\x00n\x00t\x00e\x00r\x00n\x00a\x00l\x00N\x00a\x00m\x00e\x00\x00\x00U\x00e\x00v\x00T\x00e\x00m\x00p\x00l\x00a\x00t\x00e\x00B\x00a\x00s\x00e\x00l\x00i\x00n\x00e\x00G\x00e\x00n\x00e\x00r\x00a\x00t\x00o\x00r\x00.\x00e\x00x\x00e\x00\x00\x00\x00\x00\x98\x00:\x00\x01\x00L\x00e\x00g\x00a\x00l\x00C\x00o\x00p\x00y\x00r\x00i\x00g\x00h\x00t\x00\x00\x00C\x00o\x00p\x00y\x00r\x00i\x00g\x00h\x00t\x00 \x00(\x00c\x00)\x00 \x00M\x00i\x00c\x00r\x00o\x00s\x00o\x00f\x00t\x00 \x00C\x00o\x00r\x00p\x00o\x00r\x00a\x00t\x00i\x00o\x00n\x00.\x00 \x00A\x00l\x00l\x00 \x00r\x00i\x00g\x00h\x00t\x00s\x00 \x00r\x00e\x00s\x00e\x00r\x00v\x00e\x00d\x00.\x00\x00\x00*\x00\x01\x00\x01\x00L\x00e\x00g\x00a\x00l\x00T\x00r\x00a\x00d\x00e\x00m\x00a\x00r\x00k\x00s\x00\x00\x00\x00\x00\x00\x00\x00\x00j\x00!\x00\x01\x00O\x00r\x00i\x00g\x00i\x00n\x00a\x00l\x00F\x00i\x00l\x00e\x00n\x00a\x00m\x00e\x00\x00\x00U\x00e\x00v\x00T\x00e\x00m\x00p\x00l\x00a\x00t\x00e\x00B\x00a\x00s\x00e\x00l\x00i\x00n\x00e\x00G\x00e\x00n\x00e\x00r\x00a\x00t\x00o\x00r\x00.\x00e\x00x\x00e\x00\x00\x00\x00\x00v\x00+\x00\x01\x00P\x00r\x00o\x00d\x00u\x00c\x00t\x00N\x00a\x00m\x00e\x00\x00\x00\x00\x00M\x00i\x00c\x00r\x00o\x00s\x00o\x00f\x00t\x00 \x00(\x00R\x00)\x00 \x00W\x00i\x00n\x00d\x00o\x00w\x00s\x00 \x00(\x00R\x00)\x00 \x00O\x00p\x00e\x00r\x00a\x00t\x00i\x00n\x00g\x00 \x00S\x00y\x00s\x00t\x00e\x00m\x00\x00\x00\x00\x004\x00\x08\x00\x01\x00P\x00r\x00o\x00d\x00u\x00c\x00t\x00V\x00e\x00r\x00s\x00i\x00o\x00n\x00\x00\x000\x00.\x000\x00.\x000\x00.\x000\x00\x00\x008\x00\x08\x00\x01\x00A\x00s\x00s\x00e\x00m\x00b\x00l\x00y\x00 \x00V\x00e\x00r\x00s\x00i\x00o\x00n\x00\x00\x000\x00.\x000\x00.\x000\x00.\x000\x00\x00\x00\xb8d\x00\x00\x93\x02\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\xef\xbb\xbf<?xml version="1.0" encoding="utf-8"?>\r\n<asmv1:assembly manifestVersion="1.0" xmlns="urn:schemas-microsoft-com:asm.v1" xmlns:asmv1="urn:schemas-microsoft-com:asm.v1" xmlns:asmv2="urn:schemas-microsoft-com:asm.v2" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">\r\n  <assemblyIdentity version="1.0.0.0" name="UevTemplateBaselineGenerator.app"/>\r\n  <trustInfo xmlns="urn:schemas-microsoft-com:asm.v2">\r\n    <security>\r\n      <requestedPrivileges xmlns="urn:schemas-microsoft-com:asm.v3">\r\n        <requestedExecutionLevel  level="requireAdministrator" uiAccess="false" />\r\n      </requestedPrivileges>\r\n    </security>\r\n  </trustInfo>\r\n</asmv1:assembly>\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00@\x00\x00\x0c\x00\x00\x00\xa09\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
    * 2
)
secrets = (
    """
 The Raven
By Edgar Allan Poe
Once upon a midnight dreary, while I pondered, weak and weary,
Over many a quaint and curious volume of forgotten lore—
    While I nodded, nearly napping, suddenly there came a tapping,
As of some one gently rapping, rapping at my chamber door.
“’Tis some visitor,” I muttered, “tapping at my chamber door—
            Only this and nothing more.”

    Ah, distinctly I remember it was in the bleak December;
And each separate dying ember wrought its ghost upon the floor.
    Eagerly I wished the morrow;—vainly I had sought to borrow
    From my books surcease of sorrow—sorrow for the lost Lenore—
For the rare and radiant maiden whom the angels name Lenore—
            Nameless here for evermore.

    And the silken, sad, uncertain rustling of each purple curtain
Thrilled me—filled me with fantastic terrors never felt before;
    So that now, to still the beating of my heart, I stood repeating
    “’Tis some visitor entreating entrance at my chamber door—
Some late visitor entreating entrance at my chamber door;—
            This it is and nothing more.”

    Presently my soul grew stronger; hesitating then no longer,
“Sir,” said I, “or Madam, truly your forgiveness I implore;
    But the fact is I was napping, and so gently you came rapping,
    And so faintly you came tapping, tapping at my chamber door,
That I scarce was sure I heard you”—here I opened wide the door;—
            Darkness there and nothing more.

    Deep into that darkness peering, long I stood there wondering, fearing,
Doubting, dreaming dreams no mortal ever dared to dream before;
    But the silence was unbroken, and the stillness gave no token,
    And the only word there spoken was the whispered word, “Lenore?”
This I whispered, and an echo murmured back the word, “Lenore!”—
            Merely this and nothing more.

    Back into the chamber turning, all my soul within me burning,
Soon again I heard a tapping somewhat louder than before.
    “Surely,” said I, “surely that is something at my window lattice;
      Let me see, then, what thereat is, and this mystery explore—
Let my heart be still a moment and this mystery explore;—
            ’Tis the wind and nothing more!”

    Open here I flung the shutter, when, with many a flirt and flutter,
In there stepped a stately Raven of the saintly days of yore;
    Not the least obeisance made he; not a minute stopped or stayed he;
    But, with mien of lord or lady, perched above my chamber door—
Perched upon a bust of Pallas just above my chamber door—
            Perched, and sat, and nothing more.

Then this ebony bird beguiling my sad fancy into smiling,
By the grave and stern decorum of the countenance it wore,
“Though thy crest be shorn and shaven, thou,” I said, “art sure no craven,
Ghastly grim and ancient Raven wandering from the Nightly shore—
Tell me what thy lordly name is on the Night’s Plutonian shore!”
            Quoth the Raven “Nevermore.”

    Much I marvelled this ungainly fowl to hear discourse so plainly,
Though its answer little meaning—little relevancy bore;
    For we cannot help agreeing that no living human being
    Ever yet was blessed with seeing bird above his chamber door—
Bird or beast upon the sculptured bust above his chamber door,
            With such name as “Nevermore.”

    But the Raven, sitting lonely on the placid bust, spoke only
That one word, as if his soul in that one word he did outpour.
    Nothing farther then he uttered—not a feather then he fluttered—
    Till I scarcely more than muttered “Other friends have flown before—
On the morrow he will leave me, as my Hopes have flown before.”
            Then the bird said “Nevermore.”

    Startled at the stillness broken by reply so aptly spoken,
“Doubtless,” said I, “what it utters is its only stock and store
    Caught from some unhappy master whom unmerciful Disaster
    Followed fast and followed faster till his songs one burden bore—
Till the dirges of his Hope that melancholy burden bore
            Of ‘Never—nevermore’.”

    But the Raven still beguiling all my fancy into smiling,
Straight I wheeled a cushioned seat in front of bird, and bust and door;
    Then, upon the velvet sinking, I betook myself to linking
    Fancy unto fancy, thinking what this ominous bird of yore—
What this grim, ungainly, ghastly, gaunt, and ominous bird of yore
            Meant in croaking “Nevermore.”

    This I sat engaged in guessing, but no syllable expressing
To the fowl whose fiery eyes now burned into my bosom’s core;
    This and more I sat divining, with my head at ease reclining
    On the cushion’s velvet lining that the lamp-light gloated o’er,
But whose velvet-violet lining with the lamp-light gloating o’er,
            She shall press, ah, nevermore!

    Then, methought, the air grew denser, perfumed from an unseen censer
Swung by Seraphim whose foot-falls tinkled on the tufted floor.
    “Wretch,” I cried, “thy God hath lent thee—by these angels he hath sent thee
    Respite—respite and nepenthe from thy memories of Lenore;
Quaff, oh quaff this kind nepenthe and forget this lost Lenore!”
            Quoth the Raven “Nevermore.”

    “Prophet!” said I, “thing of evil!—prophet still, if bird or devil!—
Whether Tempter sent, or whether tempest tossed thee here ashore,
    Desolate yet all undaunted, on this desert land enchanted—
    On this home by Horror haunted—tell me truly, I implore—
Is there—is there balm in Gilead?—tell me—tell me, I implore!”
            Quoth the Raven “Nevermore.”

    “Prophet!” said I, “thing of evil!—prophet still, if bird or devil!
By that Heaven that bends above us—by that God we both adore—
    Tell this soul with sorrow laden if, within the distant Aidenn,
    It shall clasp a sainted maiden whom the angels name Lenore—
Clasp a rare and radiant maiden whom the angels name Lenore.”
            Quoth the Raven “Nevermore.”

    “Be that word our sign of parting, bird or fiend!” I shrieked, upstarting—
“Get thee back into the tempest and the Night’s Plutonian shore!
    Leave no black plume as a token of that lie thy soul hath spoken!
    Leave my loneliness unbroken!—quit the bust above my door!
Take thy beak from out my heart, and take thy form from off my door!”
            Quoth the Raven “Nevermore.”

    And the Raven, never flitting, still is sitting, still is sitting
On the pallid bust of Pallas just above my chamber door;
    And his eyes have all the seeming of a demon’s that is dreaming,
    And the lamp-light o’er him streaming throws his shadow on the floor;
And my soul from out that shadow that lies floating on the floor
            Shall be lifted—nevermore!
""".encode()
    * 5
)


def generate_entropy_encoding() -> Tuple[bytearray, Dict[int, bytearray]]:
    """
    This function generates an encoding to bypass entropy checks.
    """

    base32_characters = bytearray(b"01234567ABCDEFGHIJKLMNOPQRSTUVWXYZ=")
    random_characters = bytearray(
        b"89abcdefghijklmnopqrstuvwxyz!\"#$%&'()*+,-./:;<>?@[\\]^_`{|}~ \t\n\r"
    )

    data = bytearray()

    encoding = defaultdict(bytearray)

    while base32_characters:
        character = choice(base32_characters)
        base32_characters.remove(character)
        data.append(character)
        encoding[character].append(character)

        choice_ = choice(random_characters)
        random_characters.remove(choice_)

        for _ in range(randint(2, 5)):
            data.append(choice_)
            encoding[character].append(choice_)

    return data + random_characters, encoding


def generate_entropy_encoding2() -> Tuple[bytearray, Dict[int, bytearray]]:
    """
    This function generates an encoding to bypass entropy checks.

     + Very difficult to identify
     - Entropy score is higher
     - Longer to encode/decode
    """

    base32_characters = bytearray(b"01234567ABCDEFGHIJKLMNOPQRSTUVWXYZ=")
    random_characters = bytearray(
        b"89abcdefghijklmnopqrstuvwxyz!\"#$%&'()*+,-./:;<>?@[\\]^_`{|}~ \t\n\r"
    )

    data = bytearray()

    encoding = defaultdict(bytearray)

    while base32_characters:
        character = choice(base32_characters)
        base32_characters.remove(character)
        data.append(character)

        choice1 = choice(random_characters)
        random_characters.remove(choice1)
        encoding[character].append(choice1)

        choice2 = choice(random_characters)
        random_characters.remove(choice2)

        order = randint(0, 1)
        if order:
            data.append(choice1)
        for _ in range(randint(2, 5)):
            data.append(choice2)
            encoding[character].append(choice2)
        if not order:
            data.append(choice1)

        random_characters.append(character)

    return data, encoding


def get_entropy_encoding(data: bytes) -> bytes:
    """
    This function returns the encoding to decode entropy-encoding.
    """

    encoding = {}
    while len(encoding) < 70:
        character1 = data[0]
        character2 = data[1]

        encoding[character1] = character1
        encoding[character2] = character1

        index = 2
        for character in data[2:]:
            if character != character2:
                data = data[index:]
                break
            index += 1

    return data[28:], encoding


def get_entropy_encoding2(data: bytes) -> bytes:
    """
    This function returns the encoding to decode entropy-encoding.
    """

    encoding = {}
    while len(encoding) < 70:
        character_base = data[0]
        character1 = data[1]
        character2 = data[2]
        index = 3

        while character1 == character2:
            character2 = data[index]
            index += 1

        encoding[character1] = character_base
        encoding[character2] = character_base

        for character in data[index:]:
            if character != character2:
                data = data[index:]
                break
            index += 1

    return data, encoding


def entropy_encode(data: bytes, version: int = 1) -> bytes:
    """
    This function encodes data to bypass entropy checks.

    version should be 1 or 2.
    """

    data_encoded, encoding = (
        generate_entropy_encoding()
        if version == 1
        else generate_entropy_encoding2()
    )
    for character in b32encode(data):
        data_encoded.append(choice(encoding[character]))

    return data_encoded


def entropy_encode2(data: bytes) -> bytes:
    """
    Call entropy_encode with version 2.
    """

    return entropy_encode(data, version=2)


def entropy_decode(data: bytes, version: int = 1) -> bytes:
    """
    This function decodes entropy-encoding to retrieve
    data from entropy-encoded data.

    version should be 1 or 2.
    """

    data, encoding = (
        get_entropy_encoding(data)
        if version == 1
        else get_entropy_encoding2(data)
    )
    data_decoded = bytearray()
    for character in data:
        data_decoded.append(encoding[character])

    return b32decode(data_decoded)


def entropy_decode2(data: bytes) -> bytes:
    """
    Call entropy_decode with version 2.
    """

    return entropy_decode(data, version=2)


def shannon_entropy(data: bytes) -> float:
    """
    This function returns the shannon entropy for bytes.

    Greater entropy = more randomness
    Max entropy: 8
    Min entropy: 0
    """

    frequency = Counter()
    for char in data:
        frequency[char] += 1

    entropy = 0
    data_length = len(data)
    for key in frequency:
        p = frequency[key] / data_length
        entropy -= p * log(p, 2)

    return entropy


def test():
    # For test i use librc4 to encrypt data to get higher entropy:
    #     - https://github.com/mauricelambert/FastRC4/releases/download/v0.0.1/librc4.dll
    #     - https://github.com/mauricelambert/FastRC4/releases/download/v0.0.1/librc4.so
    #     - https://raw.githubusercontent.com/mauricelambert/FastRC4/main/librc4.py

    from urllib.request import urlopen
    from shutil import copyfileobj
    from os import remove, name

    with open("librc4.dll", "wb") as file:
        copyfileobj(
            urlopen(
                "https://github.com/mauricelambert/FastRC4/releases/download/v0.0.1/librc4.dll"
            ),
            file,
        )

    with open("librc4.so", "wb") as file:
        copyfileobj(
            urlopen(
                "https://github.com/mauricelambert/FastRC4/releases/download/v0.0.1/librc4.so"
            ),
            file,
        )

    with open("librc4.py", "wb") as file:
        copyfileobj(
            urlopen(
                "https://raw.githubusercontent.com/mauricelambert/FastRC4/main/librc4.py"
            ),
            file,
        )

    from librc4 import RC4

    rc4 = RC4(b"my secret key")
    print("Entropy for non-encoded secrets:", shannon_entropy(secrets))
    encrypted_data = rc4.encrypt(secrets)
    print(
        "Entropy for non-encoded encrypted secrets:",
        shannon_entropy(encrypted_data),
    )
    data = entropy_encode(encrypted_data)
    print(
        "Entropy for entropy-encoded encrypted secrets:", shannon_entropy(data)
    )
    assert encrypted_data == entropy_decode(data)
    data = entropy_encode2(encrypted_data)
    print(
        "Entropy for entropy-encoded2 encrypted secrets:",
        shannon_entropy(data),
    )
    assert encrypted_data == entropy_decode2(data)

    rc4 = RC4(b"my secret key")
    print("Entropy for non-encoded exe:", shannon_entropy(dll))
    encrypted_data = rc4.encrypt(dll)
    print(
        "Entropy for non-encoded encrypted exe:",
        shannon_entropy(encrypted_data),
    )
    data = entropy_encode(encrypted_data)
    print("Entropy for entropy-encoded encrypted exe:", shannon_entropy(data))
    assert encrypted_data == entropy_decode(data)
    data = entropy_encode2(encrypted_data)
    print("Entropy for entropy-encoded2 encrypted exe:", shannon_entropy(data))
    assert encrypted_data == entropy_decode2(data)

    if name == "nt":
        from ctypes import windll

        windll.kernel32.FreeLibrary(".\\librc4.dll")

    remove("librc4.py")
    remove("librc4.so")
    remove("librc4.dll")


if __name__ == "__main__":
    test()
